#define DEBUGGING_ENABLED false
#define CV_INPUT_ENABLED false

#include <MozziGuts.h>
#include <Oscil.h> // oscillator 
#include <tables/sin2048_int8.h>
#include <tables/smoothsquare8192_int8.h>
#include <tables/cos8192_int8.h>
#include <tables/cos2048_int8.h>
#include <tables/whitenoise8192_int8.h>
#include <Smooth.h>
#include <AutoMap.h> // maps unpredictable inputs to a range

#define CONTROL_RATE 1024
 
// desired carrier frequency max and min, for AutoMap
const int MIN_CARRIER_FREQ = 22;
const int MAX_CARRIER_FREQ = 440;

// desired intensity max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_INTENSITY = 700;
const int MAX_INTENSITY = 10;

// desired mod speed max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_MOD_SPEED = 10000;
const int MAX_MOD_SPEED = 1;

AutoMap kMapCarrierFreq(0,1023,MIN_CARRIER_FREQ,MAX_CARRIER_FREQ);
AutoMap kMapIntensity(0,1023,MIN_INTENSITY,MAX_INTENSITY);
AutoMap kMapModSpeed(0,1023,MIN_MOD_SPEED,MAX_MOD_SPEED);

#if CV_INPUT_ENABLED
const int CV1_PIN = 0; // set the input for the knob to analog pin 0
const int CV2_PIN = 1;
const int CV3_PIN = 2;
#endif

#if CV_INPUT_ENABLED == false
const int CV1_PIN = 2;
#endif

const int LDR1_PIN=3; // set the analog input for fm_intensity to pin 1
const int LDR2_PIN=4; // set the analog input for mod rate to pin 2
const char volume_PIN = 5;

Oscil<SIN2048_NUM_CELLS, AUDIO_RATE> aModulator(SIN2048_DATA);
Oscil <SIN2048_NUM_CELLS, AUDIO_RATE> aSin(SIN2048_DATA);

Oscil <COS8192_NUM_CELLS, CONTROL_RATE> kModFreq1(COS8192_DATA);
Oscil<SIN2048_NUM_CELLS, CONTROL_RATE> kIntensityMod(SIN2048_DATA);

Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aCarrier(COS2048_DATA);
Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aModulator2(COS2048_DATA);

Oscil <2048, AUDIO_RATE> mainVCO;
Oscil <8192, AUDIO_RATE> VCO2;

int mod_ratio = 5; // brightness (harmonics)
long fm_intensity; // carries control info from updateControl to updateAudio
byte volume;
int waveNumber = 0;

// smoothing for intensity to remove clicks on transitions
float smoothness = 0.95f;
Smooth <long> aSmoothIntensity(smoothness);


void setup(){
  #if DEBUGGING_ENABLED
  //Serial.begin(9600); // for Teensy 3.1, beware printout can cause glitches
  Serial.begin(115200); // set up the Serial output so we can look at the piezo values // set up the Serial output so we can look at the light level
  #endif

  kModFreq1.setFreq(0.95f);
  startMozzi(); // :))

}


void updateControl(){
  // read the knob
  int cv1_value = mozziAnalogRead(CV1_PIN); // value is 0-1023
  
  int volume_value = mozziAnalogRead(volume_PIN);

  #if CV_INPUT_ENABLED
  int cv2_value = mozziAnalogRead(CV2_PIN);
  int cv3_value = mozziAnalogRead(CV3_PIN);
  Serial.print(cv1_value);
  Serial.print("\t");
  Serial.print(cv2_value);
  Serial.print("\t");
  Serial.print(cv3_value);
  Serial.print("\t");
  #endif
    
  // map the knob to carrier frequency
  int carrier_freq = kMapCarrierFreq(cv1_value);
  
  
  //calculate the modulation frequency to stay in ratio
  int mod_freq = carrier_freq * mod_ratio;

  // set the FM oscillator frequencies
   #if CV_INPUT_ENABLED == false
  aSin.setFreq(carrier_freq + 0.4313f*kModFreq1.next()); 
  aModulator.setFreq(mod_freq);
  #endif

  #if CV_INPUT_ENABLED        /////////////////////////////////
    waveNumber = map(cv3_value,0,1023, 0, 1);
  switch (waveNumber)
  {
    case 0:
      mainVCO.setTable(SIN2048_DATA);
      break;
    case 1:
      VCO2.setTable(SMOOTHSQUARE8192_DATA);
      break;
  }
  Serial.print(waveNumber);

  
  
  mainVCO.setFreq(carrier_freq + 0.4313f*kModFreq1.next()); 
  aModulator.setFreq(mod_freq);

  #endif
  
  // read the light dependent resistor on the width Analog input pin
  int LDR1_value= mozziAnalogRead(LDR1_PIN); // value is 0-1023

  volume= map(volume_value, 0, 1023, 0, 255);


  #if DEBUGGING_ENABLED
  // print the value to the Serial monitor for debugging
  Serial.print("LDR1 = "); 
  Serial.print(LDR1_value);
  Serial.print("\t"); // prints a tab
  #endif
  
  int LDR1_calibrated = kMapIntensity(LDR1_value);

  #if CV_INPUT_ENABLED
  int LDR2_calibrated = kMapIntensity(cv3_value);
  #endif
  
  #if DEBUGGING_ENABLED
  Serial.print("LDR1_calibrated = ");
  Serial.print(LDR1_calibrated);
  Serial.print("\t"); // prints a tab
  #endif
  
 // calculate the fm_intensity
  fm_intensity = ((long)LDR1_calibrated * (kIntensityMod.next()+128))>>8; // shift back to range after 8 bit multiply
  
  #if DEBUGGING_ENABLED
  Serial.print("fm_intensity = ");
  Serial.print(fm_intensity);
  Serial.print("\t"); // prints a tab
  #endif
  
  // read the light dependent resistor on the speed Analog input pin
  int LDR2_value= mozziAnalogRead(LDR2_PIN); // value is 0-1023
  #if DEBUGGING_ENABLED
  Serial.print("LDR2 = "); 
  Serial.print(LDR2_value);
  Serial.print("\t"); // prints a tab
  #endif
  
  // use a float here for low frequencies
  float mod_speed = (float)kMapModSpeed(LDR2_value)/30;
  
  #if DEBUGGING_ENABLED
  Serial.print("   mod_speed = ");
  Serial.print(mod_speed);
  Serial.print("\t"); // prints a tab
  #endif
  
  kIntensityMod.setFreq(mod_speed);
  
  #if DEBUGGING_ENABLED
  Serial.print("volume = ");
  Serial.print((int)volume);
  
  Serial.println(); // finally, print a carraige return for the next line of debugging info
  #endif

}



int updateAudio(){
  #if CV_INPUT_ENABLED == false
   long modulation = (aSmoothIntensity.next(fm_intensity) * aModulator.next());
  return (aSin.phMod(modulation) * volume)>>8;
  #endif

  #if CV_INPUT_ENABLED
  if (waveNumber < 1){
   long modulation = (aSmoothIntensity.next(fm_intensity) * (aModulator.next() + aModulator2.next()));
  return (mainVCO.phMod(modulation) * volume)>>8;
  } else {
    long modulation = (aSmoothIntensity.next(fm_intensity) * (aModulator.next() + aModulator2.next()));
  return (VCO2.phMod(modulation) * volume)>>8;
  }
  #endif
}


void loop(){
  audioHook();
}





